@@ -31,10 +31,11 @@ require Exporter;
 use warnings;
 use strict;
 
-our $VERSION = '0.002';
+our $VERSION = '0.003';
 
 use Carp;
 use File::Spec;
+use File::Versions 'make_backup';
 
 =head1 DESCRIPTION
 
@@ -88,6 +89,29 @@ sub convert_to_c_string
     return $text;
 }
 
+=head2 ch_files
+
+Make a .h filename from a .c filename. Back up both C and .h files.
+
+=cut
+
+sub ch_files
+{
+    my ($c_file_name) = @_;
+    if ($c_file_name !~ /\.c/) {
+       die "$c_file_name is not a C file name";
+    }
+    my $h_file_name = $c_file_name;
+    $h_file_name =~ s/\.c$/\.h/;
+    if (-f $c_file_name) {
+	make_backup ($c_file_name);
+    }
+    if (-f $h_file_name) {
+	make_backup ($h_file_name);
+    }
+    return $h_file_name;
+}
+
 =head2 convert_to_c_pc
 
     my $c_string = convert_to_c_pc ($string);     
@@ -187,7 +211,9 @@ sub wrapper_name
 {
     my ($string) = @_;
     $string =~ s/[.-]/_/g;
-    die "Bad string '$string'" unless valid_c_variable ($string);
+    if (! valid_c_variable ($string)) {
+        croak "Bad string for wrapper '$string'";
+    }
     my $wrapper_name = uc $string;
     return $wrapper_name;
 }
@@ -214,10 +240,22 @@ sub print_top_h_wrapper
 {
     my ($fh, $file_name) = @_;
     my $wrapper_name = wrapper_name ($file_name);
-    print $fh <<EOF;
+    my $wrapper = <<EOF;
 #ifndef $wrapper_name
 #define $wrapper_name
 EOF
+    print_out ($fh, $wrapper);
+}
+
+sub print_out
+{
+    my ($fh, $wrapper) = @_;
+    if (ref $fh && ref $fh eq 'SCALAR') {
+        ${$fh} .= $wrapper;
+    }
+    else {
+        print $fh $wrapper;
+    }
 }
 
 =head2 print_bottom_h_wrapper
@@ -238,9 +276,10 @@ sub print_bottom_h_wrapper
 {
     my ($fh, $file_name) = @_;
     my $wrapper_name = wrapper_name ($file_name);
-    print $fh <<EOF;
+    my $wrapper = <<EOF;
 #endif /* $wrapper_name */
 EOF
+    print_out ($fh, $wrapper);
 }
 
 =head2 print_include
@@ -313,11 +352,11 @@ sub hash_to_c_file
     print_top_h_wrapper ($h_out, $h_file_name);
     for my $variable (sort keys %$hash_ref) {
 	if (!valid_c_variable ($variable)) {
-	    die "bad variable $variable";
+	    croak "key '$variable' is not a valid C variable";
 	}
 	my $value = $hash_ref->{$variable};
-	$value = escape_string ($value);
-	print $c_out "const char * $prefix$variable = \"$value\";\n";
+	$value = convert_to_c_string ($value);
+	print $c_out "const char * $prefix$variable = $value;\n";
 	print $h_out "extern const char * $prefix$variable; /* $value */\n";
     }
     close $c_out or die $!;
@@ -343,7 +382,7 @@ sub line_directive
     my ($output, $line_number, $file_name) = @_;
     die "$line_number is not a real line number"
 	unless $line_number =~ /^\d+$/;
-    print $output "#line $line_number \"$file_name\"\n";
+    print_out ($output, "#line $line_number \"$file_name\"\n");
 }
 
 =head2 brute_force_line
